#!/bin/bash
# Check for failure of a live stream and play the backup file if live unavalable,
# reverting to the live stream within one minute of the stream coming back online.
#
# Since cron scripts run on the minute, wait 10 seconds to run just after the other check.
# This will help avoid any possible clash with the mplayer check script.
sleep 10
#
# Get the selected live source url from the database.
sourceurl=`mysql -uroot -pchooseapassword stl --skip-column-names -e "select url from live"`
sourcename=`mysql -uroot -pchooseapassword stl --skip-column-names -e "select sourcename from live"`
sourcevolume=`mysql -uroot -pchooseapassword stl --skip-column-names -e "select volume from live"`
backupstatus=`mysql -uroot -pchooseapassword stl --skip-column-names -e "select backupstatus from live"`
backupfile=`mysql -uroot -pchooseapassword stl --skip-column-names -e "select url from sources WHERE source='BACKUP'"`
backupvolume=`mysql -uroot -pchooseapassword stl --skip-column-names -e "select volume from sources WHERE source='BACKUP'"`
#
#Debug Code
#echo Source URL: $sourceurl - This is the location or file that SHOULD be live.
#echo Source Name: $sourcename - The interla name - usually a number - of this source.
#echo Backup Status: $backupstatus - OK=Normal/FAIL=Backup playback already in progress.
#echo Backup File: $backupfile - The file we should play as a backup, if the source URL is a network feed and it becomes unavailable.
#
#
# Is a local file selected as the live source? If it is then no network check is required as it will play and on end restart after a minute.
# This might be used to play a long programm direct from the transmitter during engineering work etc.
# Check for local file by ensuring that the string http:// is in sourceurl (we can only check http sources, any others are unchecked)!
# If it is not we terminate as this script is not needed!
searchString="http://"
if `echo ${sourceurl} | grep "${searchString}" 1>/dev/null 2>&1`
then
  #echo "This is a remote URL so we continue below"
  # Sleep is just to stop the script erroring when the echo line is commented out!
  sleep 1
else
  #echo "Local file selected so terminate this script."
  exit 0
fi
#
#
# If current $backupstatus is OK then we check it.
if [ "$backupstatus" = "OK" ]
then
  #echo "Backupstatus OK"
  #
  # Backup status is OK. Check sourceurl and confirm accessible.
  # We use CURL to check the sourceurl with / test as a suffix to avoid acccessing the real url and then check
  # that the response contains "HTTP" (we ignore anything else as it may vary depending on server type).
  # If HTTP is not seen we know we have a problem.
  #
  # Important: This only checks that the remote server is available, not that any stream is present or valid!
  # If the stream source goes offline but the server is still up, then this will not be detected. This script is
  # intended to give protection against a network connectivity failure to a transmitter site. You will still need
  # to manually switch sources and use a seperate silence detector to prevent such failures causing dead air!
  #
  response=$(curl --connect-time 5 -Is $sourceurl/test | head -n 1)
  searchString="HTTP"
  if `echo ${response} | grep "${searchString}" 1>/dev/null 2>&1`
  then
    # Source is ok. Exit.
    # LEDs run on, fail off.
    echo "1" > /sys/class/gpio/gpio17/value
    echo "0" > /sys/class/gpio/gpio18/value
    exit 0
    #echo "Source is OK!"
  fi
  #echo "Source not ok 1"
  # LEDs run off, fail on.
    echo "0" > /sys/class/gpio/gpio17/value
    echo "1" > /sys/class/gpio/gpio18/value
  # Test again in case this was a one-off failure.
  response=$(curl --connect-time 5 -Is $sourceurl/test | head -n 1)
  searchString="HTTP"
  if `echo ${response} | grep "${searchString}" 1>/dev/null 2>&1`
  then
    # Source is ok. Exit.
    # LEDs run on, fail off.
    echo "1" > /sys/class/gpio/gpio17/value
    echo "0" > /sys/class/gpio/gpio18/value
    exit 0
    #echo "Source is OK!"
  fi
  #echo "Source not ok 2"
  # Test a final third time to be sure!
  response=$(curl --connect-time 5 -Is $sourceurl/test | head -n 1)
  searchString="HTTP"
  if `echo ${response} | grep "${searchString}" 1>/dev/null 2>&1`
  then
    # Source is ok. Exit.
    # LEDs run on, fail off.
    echo "1" > /sys/class/gpio/gpio17/value
    echo "0" > /sys/class/gpio/gpio18/value
    exit 0
    #echo "Source is OK!"
  fi
  echo "Source not ok after checking 3 times"
  #
  # The URL or network is down! Source or network failure detected.
#echo "Network Link Down!"
  #	Set the backupstatus database flag to "FAIL".
  sqlout=`mysql -uroot -pchooseapassword stl --skip-column-names -e "UPDATE live SET backupstatus='FAIL'"`
  #
  # Log FAIL event.
  timesmp=$(date "+%Y-%m-%d %H:%M:%S")
  sqlout=`mysql -uroot -pchooseapassword stl --skip-column-names -e "INSERT INTO logs (timestamp, event) VALUES ('$timesmp','FAIL $sourceurl')"`
  #
  # Kill any existing mplayer
  killall -9 mplayer
  #
  # Set mplayer to play the backup file, with the repeat option set (-loop 0).
  #
  # Note that this will play the backup file until mplayer stops, at which point the mplayer check script will
  # again continue try to play the live stream and should reset this to OK before trying. If this then fails this script should again take over within a minute.
  #
  # Note: We don't output the metadata from mplayer as we don't want to advertise that we're running on backup! :-)
  rm /tmp/mplayer.log
  amixer -c 0 sset PCM,0 $backupvolume% </dev/null >/dev/null 2>&1 &
  mplayer -loop 0 -ao alsa:device=hw=0.0 $backupfile </dev/null >/dev/null 2>&1 &
  # LEDs run on, fail on.
  echo "1" > /sys/class/gpio/gpio17/value
  echo "1" > /sys/class/gpio/gpio18/value
  exit 0
  #
  #
else
  #
  # Current status is FAIL.
  #
  # Note that we might already be live at this point with FAIL set, (ie if the backup file ended or another source was selected), so we have to check for this.
  # Check for recovery of network.
  # Connect to live url and confirm if ok:
  #
  response=$(curl --connect-time 5 -Is $sourceurl/test | head -n 1)
  searchString="HTTP"
  if `echo ${response} | grep "${searchString}" 1>/dev/null 2>&1`
  then
    # URL source is ok.
    # This should only happen once when in recovery as we set the "OK" flag to prevent reoccurance:
    #
    # Log OK event.
    timesmp=$(date "+%Y-%m-%d %H:%M:%S")
    sqlout=`mysql -uroot -pchooseapassword stl --skip-column-names -e "INSERT INTO logs (timestamp, event) VALUES ('$timesmp','OK $sourceurl')"`
    #
    # Set backup flag to "OK".
    sqlout=`mysql -uroot -pchooseapassword stl --skip-column-names -e "UPDATE live SET backupstatus='OK'"`
    #
    # Kill any existing mplayer
    killall -9 mplayer
    #
    # Play URL
    amixer -c 0 sset PCM,0 $sourcevolume% </dev/null >/dev/null 2>&1 &
    #mplayer -ao alsa:device=hw=0.0 $sourceurl </dev/null >/dev/null 2>&1 &
    mplayer -ao alsa:device=hw=0.0 $sourceurl -quiet </dev/null >/tmp/mplayer.log 2>&1 &
    #
    # LEDs run on, fail off.
    echo "1" > /sys/class/gpio/gpio17/value
    echo "0" > /sys/class/gpio/gpio18/value
    exit 0
    #
  else
    #
    # The source is still not ok. We are still in FAIL mode and the source is still down.
    #
    # Is mplayer running? If it's not running restart it with the backup file.
    if ps -ef | grep 'mplayer' | grep 'alsa' ; then
      # Mplayer is running
      sleep 1
      exit 0
    else
      # Mplayer is not running. We need to start it!
      amixer -c 0 sset PCM,0 $backupvolume% </dev/null >/dev/null 2>&1 &
      mplayer -loop 0 -ao alsa:device=hw=0.0 $backupfile </dev/null >/dev/null 2>&1 &
      exit 0
      #
    fi
  fi
fi
